/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alessandro Pozzi <signor.hyde@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import "../common/ClockUtils.js" as Utils
import "AlarmScript.js" as AScript
import "../common"

Page {
    id: alarmPage

    property real hours: 0;
    property real minutes: 0;
    property real hourHandValue: hours;
    property real minuteHandValue: minutes;
    property real perRadius: 6
    property real oldhours: 0;
    property bool timerOn: true;
    property int minuteTime: 0;
    property int minuteValue: 0;
    property int hourTime: 0;
    property int hourValue: 0;
    property int morehours: 0;
    property string ampm: "AM";

    Component.onCompleted: {
        AScript.log("AlarmPage loaded");
        timerOn = true;
        circleMinutes.state = "NORMAL";
    }

    AlarmStorage {
        id: storageAlarm
    }

    // Timer function called by the main clock loop
    function onTimerUpdate () {
        if (timerOn == true) {
            AScript.checkAlarm();
            AScript.getFirstAlarm();
        }
    }

    anchors.margins: units.gu(2)

    // A flickable qml element to accomodate the space requirement due to the amount of alarm options.
    Flickable {
        id: flickable

        clip: true
        anchors.fill: parent
        width: parent.width
        contentWidth: width; contentHeight: itemAlarm.height

        Item {
            id: itemAlarm

            width: parent.width; height: units.gu(125);

            // Row displaying the current active alarm time (00:00)
            Row {
                id: topAlarmRow

                spacing: units.gu(0.3)
                anchors { horizontalCenter: parent.horizontalCenter; top: parent.top; topMargin: units.gu(4) }

                Label {
                    id: hourslabelTimer

                    color: Theme.palette.normal.baseText
                    fontSize: "x-large"
                    text: "00"
                    visible: true
                }

                Label {
                    id: colonTimer1

                    color: Theme.palette.normal.baseText
                    visible: true
                    fontSize: "x-large"
                    text: ":"
                }

                Label {
                    id: minuteslabelTimer

                    color: Theme.palette.normal.baseText
                    fontSize: "x-large"
                    text: "00"
                }
            }

            // Label to indicate which days the current alarm is active on (Mon-Fri, Sat-Sun, etc)
            Label {
                id: typelabelAlarm

                anchors { horizontalCenter: parent.horizontalCenter; top: topAlarmRow.bottom; topMargin: units.gu(1) }
                color: Theme.palette.normal.baseText
                fontSize: "medium"
                text: ""
            }

            Item {
                id: clockAlarm

                width: parent.width; height: units.gu(32);
                anchors { top: topAlarmRow.bottom; topMargin: units.gu(6) }

                // Analog alarm clock face with clock markers at different locations.
                AnalogFaceBase {
                    id: clockOuterCircle

                    anchors.centerIn: parent
                    // Implements touch support
                    MouseArea {
                        id: minutebackmousearea

                        property real truex: mouseX - clockOuterCircle.width/2
                        property real truey: clockOuterCircle.height/2 - mouseY
                        property real angle: Math.atan2(truex, truey)
                        property real strictangle: parseInt(angle * 180 / Math.PI)
                        property real modulo: strictangle % 6

                        enabled: true
                        anchors.fill: parent
                        preventStealing:   if(circleMinutes.state == "DONE" || circleMinutes.state == "UPDATE") {
                                               true
                                           } else {
                                               false
                                           }
                        onPositionChanged:  if(circleMinutes.state == "DONE" || circleMinutes.state == "UPDATE") {
                                                if (minutebackmousearea.angle < 0) {
                                                    minuteHand.rotationAngle = strictangle - modulo + 360;
                                                    minuteValue = parseInt(minuteHand.rotationAngle/6);
                                                    minuteslabelTimer.text = Utils.zeroleft(minuteValue, 2);
                                                    minutes = minuteValue;
                                                }  else {
                                                    minuteHand.rotationAngle = strictangle - modulo + 6;
                                                    minuteValue = parseInt(minuteHand.rotationAngle/6);
                                                    minuteslabelTimer.text = Utils.zeroleft(minuteValue, 2);
                                                    minutes = minuteValue;
                                                }
                                            }
                    }
                }

                AnalogHand {
                    id: minuteHand

                    rotationAngle: parseInt(minuteHandValue) * 6
                    widthhand: units.gu(1); heighthand: units.gu(15); heighttop: units.gu(7.5)
                    colorhand: UbuntuColors.ubuntuOrange
                    colortop: Theme.palette.normal.baseText
                }

                Rectangle {
                    id: circleMinutes

                    width: units.gu(15); height: units.gu(15)
                    anchors.centerIn: parent
                    color: Theme.palette.normal.baseText
                    radius: width / 2
                    antialiasing: true

                    states:[
                        State {
                            name: "NORMAL"

                            PropertyChanges { target: clockOuterCircle; width: units.gu(30); height: units.gu(30) }
                            PropertyChanges { target: clockAlarm; anchors { top: topAlarmRow.bottom; topMargin: units.gu(6) } }
                            PropertyChanges { target: circleMinutes; width: units.gu(15); height: units.gu(15) }
                            PropertyChanges { target: minuteHand; visible: true }
                            PropertyChanges { target: hourHand; visible: true }
                            PropertyChanges { target: clockOuterCircle; visible: true }
                            PropertyChanges { target: clockAlarm; visible: true }
                            PropertyChanges { target: circleMinutes; visible: true }
                            PropertyChanges { target: buttonDone; visible: false }
                            PropertyChanges { target: alarmSwitch; visible: true }
                            PropertyChanges { target: alarmOptions; visible: false }
                            PropertyChanges { target: shapeNextAlarm; visible: true }
                            PropertyChanges { target: listAlarmSave; visible: false }
                            PropertyChanges { target: toolbarAlarm.back; visible: false }
                            PropertyChanges { target: editAlarm; visible: true }
                            PropertyChanges { target: toolbarAlarm; opened: false }
                            PropertyChanges { target: toolbarAlarm; locked: false }
                            PropertyChanges { target: flickable; interactive: false }
                            PropertyChanges { target: typelabelAlarm; visible: true }
                        },

                        State {
                            name: "DONE"

                            PropertyChanges { target: minuteHand; widthhand: units.gu(1); heighthand: units.gu(17); heighttop: units.gu(6); colorhand: Theme.palette.normal.baseText; colortop: Theme.palette.normal.baseText }
                            PropertyChanges { target: hourHand; widthhand: units.gu(1); heighthand: units.gu(11); heighttop: units.gu(6); colorhand: Theme.palette.normal.baseText; colortop: Theme.palette.normal.baseText  }
                            PropertyChanges { target: clockOuterCircle; width: units.gu(34); height: units.gu(34) }
                            PropertyChanges { target: clockAlarm; anchors { top: topAlarmRow.bottom; topMargin: units.gu(4) } }
                            PropertyChanges { target: circleMinutes; width: units.gu(22); height: units.gu(22) }
                            PropertyChanges { target: alarmSwitch; visible: false }
                            PropertyChanges { target: alarmOptions; visible: true }
                            PropertyChanges { target: shapeNextAlarm; visible: false }
                            PropertyChanges { target: listAlarmSave; visible: false }
                            PropertyChanges { target: toolbarAlarm.back; visible: true }
                            PropertyChanges { target: editAlarm; visible: true }
                            PropertyChanges { target: toolbarAlarm; opened: false }
                            PropertyChanges { target: toolbarAlarm; locked: false }
                            PropertyChanges { target: flickable; interactive: true }
                            PropertyChanges { target: typelabelAlarm; visible: false }
                            PropertyChanges { target: hourHand; rotationAngle: 0 }
                            PropertyChanges { target: minuteHand; rotationAngle: 0 }
                            PropertyChanges { target: alarmPage; hourValue: 0 }
                            PropertyChanges { target: alarmPage; minuteValue: 0 }
                            PropertyChanges { target: accordionIcon; state: "closed" }
                            PropertyChanges { target: checkMon; checked: false }
                            PropertyChanges { target: checkTue; checked: false }
                            PropertyChanges { target: checkWed; checked: false }
                            PropertyChanges { target: checkThu; checked: false }
                            PropertyChanges { target: checkFri; checked: false }
                            PropertyChanges { target: checkSun; checked: false }
                            PropertyChanges { target: checkSat; checked: false }
                            PropertyChanges { target: listTone; selectedIndex: 0 }
                            PropertyChanges { target: vibrateSwitch; checked: false }
                            PropertyChanges { target: minuteHand; visible: true }
                            PropertyChanges { target: hourHand; visible: true }
                            PropertyChanges { target: clockOuterCircle; visible: true}
                            PropertyChanges { target: clockAlarm; visible: true }
                            PropertyChanges { target: circleMinutes; visible: true }
                            PropertyChanges { target: buttonDone; visible: true }
                        },

                        State {
                            name: "UPDATE"

                            PropertyChanges { target: minuteHand; widthhand: units.gu(1); heighthand: units.gu(17); heighttop: units.gu(6); colorhand: Theme.palette.normal.baseText; colortop: Theme.palette.normal.baseText }
                            PropertyChanges { target: hourHand; widthhand: units.gu(1); heighthand: units.gu(11); heighttop: units.gu(6); colorhand: Theme.palette.normal.baseText; colortop: Theme.palette.normal.baseText  }
                            PropertyChanges { target: clockOuterCircle; width: units.gu(34); height: units.gu(34) }
                            PropertyChanges { target: clockAlarm; anchors { top: topAlarmRow.bottom; topMargin: units.gu(4) } }
                            PropertyChanges { target: circleMinutes; width: units.gu(22); height: units.gu(22) }
                            PropertyChanges { target: minuteHand; visible: true }
                            PropertyChanges { target: hourHand; visible: true }
                            PropertyChanges { target: clockOuterCircle; visible: true}
                            PropertyChanges { target: clockAlarm; visible: true }
                            PropertyChanges { target: circleMinutes; visible: true }
                            PropertyChanges { target: buttonDone; visible: true }
                            PropertyChanges { target: alarmSwitch; visible: false }
                            PropertyChanges { target: alarmOptions; visible: true }
                            PropertyChanges { target: shapeNextAlarm; visible: false }
                            PropertyChanges { target: listAlarmSave; visible: false }
                            PropertyChanges { target: toolbarAlarm.back; visible: true }
                            PropertyChanges { target: editAlarm; visible: true }
                            PropertyChanges { target: toolbarAlarm; opened: false }
                            PropertyChanges { target: toolbarAlarm; locked: false }
                            PropertyChanges { target: flickable; interactive: true }
                            PropertyChanges { target: typelabelAlarm; visible: false }
                            PropertyChanges { target: accordionIcon; state: "closed" }
                        },

                        State {
                            name: "LISTALARM"

                            PropertyChanges { target: minuteHand; visible: false }
                            PropertyChanges { target: hourHand; visible: false }
                            PropertyChanges { target: clockOuterCircle; visible: false }
                            PropertyChanges { target: clockAlarm; visible: false }
                            PropertyChanges { target: circleMinutes; visible: false }
                            PropertyChanges { target: buttonDone; visible: false }
                            PropertyChanges { target: alarmSwitch; visible: false }
                            PropertyChanges { target: alarmOptions; visible: false}
                            PropertyChanges { target: listAlarmSave; visible: true }
                            PropertyChanges { target: toolbarAlarm.back; visible: true }
                            PropertyChanges { target: editAlarm; visible: false }
                            PropertyChanges { target: toolbarAlarm; opened: true }
                            PropertyChanges { target: toolbarAlarm; locked: true }
                            PropertyChanges { target: flickable; interactive: false }
                            PropertyChanges { target: shapeNextAlarm; anchors.top: parent.top; anchors.topMargin: units.gu(12) }
                            PropertyChanges { target: typelabelAlarm; visible: false }
                        }
                    ]

                    // Adds subtle animations to the analogue alarm face
                    transitions: [
                        Transition {
                            to: "DONE"; reversible: true
                            ParallelAnimation {
                                NumberAnimation { target: clockOuterCircle; property: "width"; duration: 1000; easing.type: Easing.InOutQuad }
                                NumberAnimation { target: clockOuterCircle; property: "height"; duration: 1000; easing.type: Easing.InOutQuad }
                                NumberAnimation { target: circleMinutes; property: "width"; duration: 1000; easing.type: Easing.InOutQuad }
                                NumberAnimation { target: circleMinutes; property: "height"; duration: 1000; easing.type: Easing.InOutQuad }
                                NumberAnimation { target: minuteHand; property: "heighthand"; duration: 1000; easing.type: Easing.InOutQuad }
                                NumberAnimation { target: minuteHand; property: "heighttop"; duration: 1000; easing.type: Easing.InOutQuad }
                                NumberAnimation { target: hourHand; property: "heighthand"; duration: 1000; easing.type: Easing.InOutQuad }
                                NumberAnimation { target: hourHand; property: "heighttop"; duration: 1000; easing.type: Easing.InOutQuad }
                                NumberAnimation { target: clockAlarm; property: "anchors.topMargin"; duration: 1000; easing.type: Easing.InOutQuad }
                            }
                        },

                        Transition {
                            to: "UPDATE"; reversible: true
                            ParallelAnimation {
                                NumberAnimation { target: clockOuterCircle; property: "width"; duration: 1000; easing.type: Easing.InOutQuad }
                                NumberAnimation { target: clockOuterCircle; property: "height"; duration: 1000; easing.type: Easing.InOutQuad }
                                NumberAnimation { target: circleMinutes; property: "width"; duration: 1000; easing.type: Easing.InOutQuad }
                                NumberAnimation { target: circleMinutes; property: "height"; duration: 1000; easing.type: Easing.InOutQuad }
                                NumberAnimation { target: minuteHand; property: "heighthand"; duration: 1000; easing.type: Easing.InOutQuad }
                                NumberAnimation { target: minuteHand; property: "heighttop"; duration: 1000; easing.type: Easing.InOutQuad }
                                NumberAnimation { target: hourHand; property: "heighthand"; duration: 1000; easing.type: Easing.InOutQuad }
                                NumberAnimation { target: hourHand; property: "heighttop"; duration: 1000; easing.type: Easing.InOutQuad }
                                NumberAnimation { target: clockAlarm; property: "anchors.topMargin"; duration: 1000; easing.type: Easing.InOutQuad }
                            }
                        }
                    ]

                    // Implements touch support
                    MouseArea {
                        id: hourbackmousearea

                        property real truex: mouseX - circleMinutes.width/2
                        property real truey: circleMinutes.height/2 - mouseY
                        property real angle: Math.atan2(truex, truey)
                        property real strictangle: parseInt(angle * 180 / Math.PI)
                        property real modulo: strictangle % 30

                        enabled: true

                        anchors.fill: parent
                        preventStealing:  if (circleMinutes.state == "DONE" || circleMinutes.state == "UPDATE") {
                                              true // move only alarm hand and ensures that the sideway motion of pages do not interrupt moving the clock hand
                                          } else {
                                              false
                                          }
                        onPositionChanged:  if (circleMinutes.state == "DONE" || circleMinutes.state == "UPDATE") {
                                                if (hourbackmousearea.angle < 0) {
                                                    oldhours = hours;
                                                    hourHand.rotationAngle = strictangle - modulo + 360;
                                                    hourValue = parseInt(hourHand.rotationAngle/30);
                                                    hourslabelTimer.text = Utils.zeroleft(hourValue + morehours, 2);
                                                    hours = hourValue;
                                                    AScript.checkampm(oldhours, hours, ampm)
                                                }  else {
                                                    oldhours = hours;
                                                    hourHand.rotationAngle = strictangle - modulo + 30;
                                                    hourValue = parseInt(hourHand.rotationAngle/30);
                                                    hourslabelTimer.text = Utils.zeroleft(hourValue + morehours, 2);
                                                    hours = hourValue;
                                                    AScript.checkampm(oldhours, hours, ampm)
                                                }
                                            }
                        onClicked: {
                            if (circleMinutes.state == "NORMAL") {
                                circleMinutes.state = "DONE";
                                AScript.showWeekday(false);
                            }
                        }
                    }

                    AnalogHand {
                        id: hourHand

                        rotationAngle: parseInt(hourHandValue) * 6
                        widthhand: units.gu(1); heighthand: units.gu(7.5); heighttop: units.gu(0)
                        colortop: UbuntuColors.Orange
                        colorhand: Theme.palette.normal.baseText
                    }

                    Rectangle {
                        id: buttonDone

                        width: units.gu(10); height: units.gu(10)
                        anchors.centerIn: parent
                        color: Theme.palette.normal.baseText
                        radius: width / 2
                        antialiasing: true
                        visible: false

                        Label {
                            x: parent.width/2 - width/2; y: parent.height/2 - height/2
                            color: Theme.palette.normal.baseText
                            fontSize: "large"

                            text: i18n.tr("DONE")
                        }

                        MouseArea {
                            anchors.fill: parent

                            onClicked: {
                                if (circleMinutes.state == "DONE") {
                                    AScript.addAlarm();
                                    circleMinutes.state = "NORMAL";
                                } else {
                                    AScript.updateAlarm(AScript.selectkey);
                                    circleMinutes.state = "NORMAL";
                                    flickable.contentY = -units.gu(9.5);
                                }
                            }
                        }
                    }
                }
            }

            // Alarm switch shown in main alarm page to indicate if there any active alarms.
            Switch {
                id: alarmSwitch
                anchors { top: clockAlarm.bottom; topMargin: units.gu(3); horizontalCenter: parent.horizontalCenter }
                checked: false
                enabled: false
            }

            // Button to show the time to the next alarm and may also be used to view the list of saved alarms
            UbuntuShape {
                id: shapeNextAlarm
                width: units.gu(25); height: units.gu(7)
                anchors { top: alarmSwitch.bottom; topMargin: units.gu(2); horizontalCenter: parent.horizontalCenter }
                color: Theme.palette.normal.baseText
                Label {
                    id: labelHeaderNextAlarm

                    width: parent.width
                    anchors { top: parent.top; topMargin: units.gu(1) }
                    horizontalAlignment: Text.AlignHCenter
                    text: i18n.tr("Time to next alarm")
                    fontSize: "small"
                    visible: false
                }
                Label {
                    id: labelNextAlarm

                    width: parent.width
                    anchors { top: labelHeaderNextAlarm.top; topMargin: units.gu(2) }
                    horizontalAlignment: Text.AlignHCenter
                    text: ""
                    fontSize: "large"
                    visible: false
                }
                Label {
                    id: labelNoAlarm

                    width: parent.width; height: parent.height
                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment: Text.AlignVCenter
                    text: i18n.tr("No alarm set")
                    fontSize: "large"
                    visible: true
                }
                MouseArea {
                    anchors.fill: parent

                    onClicked: {
                        if (circleMinutes.state == "NORMAL") {
                            AScript.populateAlarm();
                            circleMinutes.state = "LISTALARM";
                        }
                    }
                }
            }

            // Provides list of alarm options like Tone, Vibration and the days to repeat the alarm on.
            Column {
                id: alarmOptions

                visible: false
                spacing: 0
                width: parent.width;
                anchors { top: clockAlarm.bottom; topMargin: units.gu(4) }

                ListItem.Divider { }

                ListItem.Standard {
                    text: i18n.tr("Repeat")

                    Label {
                        fontSize: "small"
                        text: { AScript.strTypeAlarm(checkMon.checked, checkTue.checked, checkWed.checked, checkThu.checked, checkFri.checked, checkSat.checked, checkSun.checked); }
                        anchors { verticalCenter: parent.verticalCenter; right: parent.right; rightMargin: units.gu(6.5) }
                    }

                    MouseArea {
                        anchors.fill: parent

                        onClicked: {
                            if (AScript.weekdaysvisible == true) {
                                accordionIcon.state = "closed";
                                AScript.showWeekday(false);
                            } else {
                                accordionIcon.state = "expanded";
                                AScript.showWeekday(true);
                            }
                        }
                    }
                    Item {
                        id: accordion
                        width: units.gu(4)
                        anchors {
                            right: parent.right
                            rightMargin: units.gu(2)
                            top: parent.top
                            bottom: parent.bottom
                        }
                        Image {
                            id: accordionIcon
                            anchors.centerIn: parent
                            // FIXME: Replace this placeholder graphic with the official one.
                            source: "../images/ListItemProgressionArrow.png"
                            opacity: enabled ? 1.0 : 0.5
                            //                            rotation: expanded ? 270 : 90
                            width: implicitWidth / 1.5
                            height: implicitHeight / 1.5

                            states: [
                                State {
                                    name: "expanded"
                                    when: selector.expanded
                                    PropertyChanges { target: accordionIcon; rotation: 270 }
                                }, State {
                                    name: "closed"
                                    when: !selector.expanded
                                    PropertyChanges { target: accordionIcon; rotation: 90 }
                                }
                            ]

                            transitions: Transition {
                                PropertyAnimation {
                                    target: accordionIcon
                                    properties: "rotation"
                                    duration: 100
                                }
                            }
                        }
                    }
                }

                // TODO: Replace the ListItem.Standard with ListItem.ValueSelector once it is possible to
                // choose multiple values using checkboxes.
                ListItem.Standard {
                    id: itemMon

                    text: i18n.tr("Mondays")
                    visible: AScript.weekdaysvisible
                    control: CheckBox {
                        id: checkMon
                        anchors.verticalCenter: parent.verticalCenter
                    }
                }

                ListItem.Standard {
                    id: itemTue

                    text: i18n.tr("Tuesdays")
                    visible: AScript.weekdaysvisible
                    control: CheckBox {
                        id: checkTue
                        anchors.verticalCenter: parent.verticalCenter
                    }
                }

                ListItem.Standard {
                    id: itemWed

                    text: i18n.tr("Wednesdays")
                    visible: AScript.weekdaysvisible
                    control: CheckBox {
                        id: checkWed
                        anchors.verticalCenter: parent.verticalCenter
                    }
                }

                ListItem.Standard {
                    id: itemThu

                    text: i18n.tr("Thursdays")
                    visible: AScript.weekdaysvisible
                    control: CheckBox {
                        id: checkThu
                        anchors.verticalCenter: parent.verticalCenter
                    }
                }

                ListItem.Standard {
                    id: itemFri

                    text: i18n.tr("Fridays")
                    visible: AScript.weekdaysvisible
                    control: CheckBox {
                        id: checkFri
                        anchors.verticalCenter: parent.verticalCenter
                    }
                }

                ListItem.Standard {
                    id: itemSat

                    text: i18n.tr("Saturdays")
                    visible: AScript.weekdaysvisible
                    control: CheckBox {
                        id: checkSat
                        anchors.verticalCenter: parent.verticalCenter
                    }
                }

                ListItem.Standard {
                    id: itemSun

                    text: i18n.tr("Sundays")
                    visible: AScript.weekdaysvisible
                    control: CheckBox {
                        id: checkSun
                        anchors.verticalCenter: parent.verticalCenter
                    }
                }

                ListItem.Divider { }

                // TODO: Connect the Tone list to the platform API to use the appropriate beep
                ListItem.ValueSelector {
                    id: listTone

                    text: i18n.tr("Tone")
                    values: ["Beep", "Beep2"]
                }

                ListItem.Divider { }

                // TODO: Connect the vibrate switch to the platform API
                ListItem.Standard {
                    text: i18n.tr("Vibrate")
                    control: Switch {
                        id: vibrateSwitch
                        anchors.verticalCenter: parent.verticalCenter
                    }
                }
            }

            // List the currently saved alarms
            Rectangle {
                id: listAlarmSave

                visible: false
                clip: true
                width: parent.width; height: parent.height - units.gu(40)
                anchors { top: parent.top; topMargin: units.gu(23) }
                color: "transparent"

                ListView {
                    id: labelListViewAlarm

                    width: parent.width; height: units.gu(4)
                    anchors.horizontalCenter: parent.Center

                    ListItem.Divider { }

                    ListItem.Header {
                        id: alarmHeader

                        text: i18n.tr("Saved Alarm")
                    }
                }

                ListView {
                    id: listViewAlarm

                    width: parent.width; height: parent.height - units.gu(54)
                    anchors.top: labelListViewAlarm.bottom
                    model: listModel
                    clip: true

                    delegate: ListItem.Standard {

                        text: Utils.zeroleft(hou, 2) + ":" + Utils.zeroleft(min, 2)
                        removable: true
                        backgroundIndicator: BackgroundAlarm {
                            id: backgroundAlarm
                            state: swipingState
                        }
                        onItemRemoved: if (backgroundAlarm.state == "SwipingRight") {
                                           storageAlarm.delAlarmValue(key);
                                       } else {
                                           AScript.editAlarm(key);
                                       }

                        Label {
                            text: {
                                var alarmText = AScript.strTypeAlarm(mon, tue, wed, thu, fri, sat, sun);
                                if (alarmText == "Never") {
                                    "";
                                } else if (alarmText) {
                                    alarmText;
                                }
                            }
                            anchors { verticalCenter: parent.verticalCenter; right: parent.right; rightMargin: units.gu(12) }
                        }
                        control:
                            Switch {
                            id: switchItem
                            checked: (act == 0) ? false : true;
                            anchors.verticalCenter: parent.verticalCenter

                            onCheckedChanged: {
                                if (checked) {
                                    storageAlarm.setStateAlarmValue(key, 1);
                                } else {
                                    storageAlarm.setStateAlarmValue(key, 0);
                                }
                            }
                        }
                    }
                }
            }

            ListModel { id: listModel }

            WorkerScript {
                id: worker
                source: "AlarmScript.js"
            }
        }
    }

    // Toolbar prviding actions like add alarm, edit alarm and back.
    tools: ToolbarItems {
        id: toolbarAlarm

        ToolbarButton {
            id: addAlarm
            objectName: "add-alarm"

            action: Action {
                iconSource: Qt.resolvedUrl("../images/add_icon.png")
                text: i18n.tr("Add Alarm")

                onTriggered: {
                    hourslabelTimer.text = "00";
                    minuteslabelTimer.text = "00";
                    circleMinutes.state = "DONE";
                    AScript.showWeekday(false);
                    flickable.contentY = -units.gu(9.5);
                }
            }
        }

        ToolbarButton {
            action: Action {
                id: editAlarm
                objectName: "edit-alarm"

                iconSource: Qt.resolvedUrl("../images/add.png")
                text: i18n.tr("Edit")

                onTriggered: {
                    circleMinutes.state = "LISTALARM";
                    AScript.populateAlarm();
                    flickable.contentY = -units.gu(9.5);
                }
            }
        }

        back: ToolbarButton {
            text: "Back";
            iconSource: Qt.resolvedUrl("../images/back_icon.png")
            visible: !addAlarm.visible;
            onTriggered: {
                circleMinutes.state = "NORMAL";
                flickable.contentY = -units.gu(9.5);
            }
        }
    }
}
