import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import QtQuick.XmlListModel 2.0

Column {
    id: worldClocks

    // Properties to store the user searched city details
    property string city: "null";
    property real lng: 0;
    property real lat: 0;
    property real rawOffSet: 0;    

    // Property to store the city searched by the user
    property string search_string: "null";

    // Properties to store the information related to api.geonames.org
    readonly property string username: "krnekhelesh";
    readonly property string search_base_url: "http://api.geonames.org/search?q=";
    readonly property string timezone_base_url: "http://api.geonames.org/timezone?lat=";
    readonly property string local_base_url: "common-cities.xml";
    readonly property int no_of_results: 5;

    property alias searchModel: searchCityModel;
    property alias index: worldList.currentIndex;

    signal clicked()

    function searchCityUrl (city) {
        return (search_base_url + search_string + "&maxRows=" + no_of_results + "&username=" + username + "&style=full");
    }

    function localCityUrl () {
        return Qt.resolvedUrl(local_base_url);
    }

    function getCityTimezoneUrl (lat, lng) {
        return (timezone_base_url + lat + "&lng=" + lng + "&username=" + username);
    }

    function clearUserSearch () {
        searchCityModel.source = localCityUrl();
        searchLabel.text = "";
        worldList.currentIndex = -1;
    }

    height: childrenRect.height;
    spacing: units.gu(2);

    // Xml model to search city online and retrieve latitude and longitude of searched city
    XmlListModel {
        id: searchCityModel;

        source: localCityUrl()
        query: "/geonames/geoname"

        XmlRole { name: "city"; query: "toponymName/string()"; isKey: true }
        XmlRole { name: "country"; query: "countryName/string()"; isKey: true }
        XmlRole { name: "lat"; query: "lat/string()"; isKey: true }
        XmlRole { name: "lng"; query: "lng/string()"; isKey: true }
        XmlRole { name: "adminName"; query: "adminName1/string()"; isKey: true}

        onSourceChanged: reload();
    }

    Row {
        id: searchBox;

        spacing: units.gu(1)
        anchors.horizontalCenter: parent.horizontalCenter;
        height: childrenRect.height;

        TextField {
            id: searchLabel

            hasClearButton: true
            placeholderText: i18n.tr("Search");
            primaryItem: Image {
                height: parent.height/1.4
                width: parent.height/1.45
                source: Qt.resolvedUrl("../images/search_icon.png")
            }

            onTextChanged: {
                if (searchLabel.text != "") {
                    search_string = searchLabel.text;
                    searchCityModel.source = searchCityUrl(search_string);
                } else {
                    searchCityModel.source = localCityUrl()
                }
            }
        }

        Button {
            id: searchButton;

            width: units.gu(12); height: searchLabel.height;            
            text: i18n.tr("Search")

            onClicked: {
                if (searchLabel.text != "") {
                    search_string = searchLabel.text;
                    searchCityModel.source = searchCityUrl(search_string);
                }
            }
        }

        ActivityIndicator {
            running: searchCityModel.status === XmlListModel.Loading
        }
    }

    ListView {
        id: worldList

        anchors { left: parent.left; right: parent.right }
        height: units.gu(50)
        model: searchCityModel
        currentIndex: -1
        clip: true;

        Component {
            id: sectionHeading
            ListItem.Header {
                Label {
                    text: section
                    anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(2) }
                    color: Theme.palette.normal.baseText
                    fontSize: "medium"
                }
            }
        }

        section.property: "city"
        section.criteria: ViewSection.FirstCharacter
        section.delegate: sectionHeading;
        section.labelPositioning: ViewSection.InlineLabels

        delegate: ListItem.Standard {
            Label {
                text: searchCityModel.status == XmlListModel.Ready ? (adminName ? (city + ", " + adminName + ", " + country) : (city + ", " + country)) : i18n.tr("Loading");
                anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(3) }
                color: Theme.palette.normal.baseText
                fontSize: "large"
            }
            selected: worldList.currentIndex == index;

            onClicked: {
                worldList.currentIndex = index;
                worldClocks.clicked()
            }
        }
    }
}
