/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1

// Qml element to customize list item swipe behavior.
Rectangle {
    id: swipeBackgroundItem

    anchors.fill: parent
    color: Theme.palette.normal.base;

    Image {
        id: swipeDeleteIcon

        property int slidingMargin: units.gu(12)

        anchors { verticalCenter: parent.verticalCenter; left: parent.left; right: parent.right }
        verticalAlignment: Image.AlignVCenter        
        fillMode: Image.Pad
    }

    Label {
        id: swipeBackgroundText

        property int slidingMargin: units.gu(3)

        anchors.fill: parent
        verticalAlignment: Text.AlignVCenter
        color: Theme.palette.normal.baseText
        fontSize: "large"
    }

    states: [
        State {
            name: "SwipingRight"
            PropertyChanges { target: swipeBackgroundText; anchors.rightMargin: swipeBackgroundText.slidingMargin
                anchors.leftMargin: 0; horizontalAlignment: Text.AlignRight; text: i18n.tr("Delete") }
            PropertyChanges { target: swipeDeleteIcon; anchors.rightMargin: swipeDeleteIcon.slidingMargin
                anchors.leftMargin: 0; horizontalAlignment: Image.AlignRight; source: Qt.resolvedUrl("../images/trash_icon.png")}
        },

        State {
            name: "SwipingLeft"
            PropertyChanges { target: swipeBackgroundText; anchors.rightMargin: 0; anchors.leftMargin: swipeBackgroundText.slidingMargin
                horizontalAlignment: Text.AlignLeft; text: i18n.tr("Delete") }
            PropertyChanges { target: swipeDeleteIcon; anchors.rightMargin: 0; source: Qt.resolvedUrl("../images/trash_icon.png")
                anchors.leftMargin: swipeDeleteIcon.slidingMargin; horizontalAlignment: Image.AlignLeft }
        }
    ]
}
