# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authored by: Renato Araujo Oliveira Filho <renato@canonical.com>
#              Nekhelesh Ramananthan <krnekhelesh@gmail.com>


class MainWindow(object):
    """An emulator class that makes it easy to interact with the clock-app."""

    def __init__(self, app):
        self.app = app

    def get_qml_view(self):
        """Get the main QML view"""
        return self.app.select_single("QQuickView")

    def get_object(self, typeName, name):
        return self.app.select_single(typeName, objectName=name)

    def get_object_by_id(self, typeName, objectId):
        return self.app.select_single(typeName, id=objectId)
        
    def get_stopwatch_tab_button(self):
        """Returns the stopwatch tab."""
        return self.app.select_single("AbstractButton", buttonIndex=3)       
        
    def get_stopwatch_label(self):
        """Returns the select for the stopwatch label"""
        return self.app.select_single("Label", objectName="labelStopwatch")
        
    def get_stopwatch_button(self):
        """Returns the select for the stopwatch button"""
        return self.app.select_single("AnalogStopwatch", objectName="buttonStopwatch")

    def get_lap_button(self):
        """Returns the select for the lap button"""
        return self.app.select_single("Button", objectName="lapButton")

    def get_reset_button(self):
        """Returns the select for the reset button"""
        return self.app.select_single("ImageButton", objectName="resetButton")        
        
    def get_laps(self):
        """Returns the select for the lap count"""
        return self.app.select_single("ListModel", objectName="laps")     

    def get_timer_tab_button(self):
        """Returns the timer tab."""
        return self.app.select_single("AbstractButton", buttonIndex=2)

    def get_toolbar_timer(self):
        """Returns the toolbar of timer tab"""
        return self.app.select_single("ToolbarItems")

    def get_toolbar_timer_button(self, button_idx):
        """Returns a button of toolbar of timer tab"""
        toolbar = self.app.select_single("ToolbarItems")
        button = toolbar.get_children()
        return button[button_idx]

    def get_toolbar_timer_add_preset_button(self):
        """Returns the button for add preset of toolbar of timer tab"""
        return self.get_toolbar_timer_button(1)

    def get_timer_name_preset(self):
        """Returns the TextField where insert the name of the preset"""
        return self.app.select_single("TextField", objectName="namePreset")

    def get_timer_hour_hand(self):
        """Returns the hour hand of clock in timer tab"""
        return self.app.select_single("AnalogTouchHand", objectName="hourHand")

    def get_timer_clock_center_label(self):
        """Returns the label in the center of clock"""
        return self.app.select_single("Label", objectName="addPresetTextDone")

    def get_preset_label_text(self):
        """Returns the label with the preset text"""
        preset = self.app.select_many("Label", objectName="presetTextLabel")
        return preset[0] #last preset, the one created by autopilot

    def get_preset_label_timer(self):
        """Returns the label with the preset timer"""
        preset = self.app.select_many("Label", objectName="presetTimerLabel")
        return preset[0] #last preset, the one created by autopilot

    def get_label_timer(self):
        """Return the label with the timer countdown"""
        return self.app.select_single("Label", objectName="labelTimer")

    def get_num_of_presets(self):
        """Returns the number of presets in the timer page."""
        timer_page = self.app.select_single("TimerPage")
        return timer_page.select_single("QQuickListView").count

    def get_first_preset_list_item(self):
        """Returns the first preset list item in the timer page."""
        return self.app.select_single("Standard", objectName="preset0")
