# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authored by: Renato Araujo Oliveira Filho <renato@canonical.com>


"""clock-app autopilot tests."""

import os.path
import os

from autopilot.input import Mouse, Touch, Pointer
from autopilot.platform import model
from autopilot.testcase import AutopilotTestCase
from autopilot.matchers import Eventually
from testtools.matchers import GreaterThan

from ubuntu_clock_app.emulators.main_window import MainWindow


class ClockAppTestCase(AutopilotTestCase):

    """A common test case class that provides several useful methods for
    clock-app tests.

    """
    if model() == 'Desktop':
        scenarios = [('with mouse', dict(input_device_class=Mouse))]
    else:
        scenarios = [('with touch', dict(input_device_class=Touch))]

    local_location = "../../ubuntu-clock-app.qml"

    def setUp(self):
        self.pointing_device = Pointer(self.input_device_class.create())
        super(ClockAppTestCase, self).setUp()
        if os.path.exists(self.local_location):
            self.launch_test_local()
        else:
            self.launch_test_installed()

    def launch_test_local(self):
        self.app = self.launch_test_application(
            "qmlscene",
            self.local_location,
            app_type='qt')

    def launch_test_installed(self):
        self.app = self.launch_test_application(
            "qmlscene",
            "/usr/share/ubuntu-clock-app/ubuntu-clock-app.qml",
            "--desktop_file_hint=/usr/share/applications/ubuntu-clock-app.desktop",
            app_type='qt')

    def get_toolbar(self):
        return self.app.select_single("Toolbar")

    #shamelessly pulled and adapted from the ubuntusdk emulator unmerged code, 6 Aug 13
    #need to migrate to ubuntusdk emulator
    def get_tabs(self):
        return self.app.select_single("TabBar")

    def switch_to_next_tab(self):
        """Open the next tab."""
        self.pointing_device.click_object(self._get_current_tab_button())
        self.pointing_device.click_object(self._get_next_tab_button())

    def _get_current_tab_button(self):
        tabs = self.get_tabs()
        #return tabs.tabs.select_many('AbstractButton', selected=True) <--- can't do this
        #clock has 8 buttons, a duplicate of each tab for some reason.
        #Sometimes 2 buttons are shown as selected. If this is the case, we want the second one.
        #Looping through them in this horrible way accomplishes our goal
        buttons = tabs.select_many('AbstractButton')
        for button in buttons:
            if button.selected == True:
                selectedButton = button
        return selectedButton

    def _get_next_tab_button(self):
        current_index = self._get_current_tab_button().buttonIndex
        next_index = (current_index + 1) % self._get_number_of_tab_buttons()
        return self._get_tab_buttons()[next_index]

    def _get_number_of_tab_buttons(self):
        return len(self._get_tab_buttons())

    def _get_tab_buttons(self):
        tabs = self.get_tabs()
        return tabs.select_many('AbstractButton')

    def move_to_stopwatch_tab(self):
        #stopwatch is 3 tabs over from clock
        #we really need better tab support to support moving to tab based on objectName
        self.switch_to_next_tab()
        self.switch_to_next_tab()
        self.switch_to_next_tab()

    def move_to_timer_tab(self):
        #timer is 2 tabs over from clock
        #we really need better tab support to support moving to tab based on objectName
        self.switch_to_next_tab()
        self.switch_to_next_tab()

    @property
    def main_window(self):
        return MainWindow(self.app)
