/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import "../common"

// Qml item to enable touch input to the AnalogClockHand component
Item {
    id: touchHand;

    z: parent.z;
    width: parent.width; height: width;
    anchors.centerIn: parent;

    /*!
      Enable/Disable rotation animation. Disable it while dragging the clock hand to avoid
      receiving inaccurate input values.
      */
    property alias animateFlag: hand.handRotationAnimation

    /*!
      Set if the dial should be rotated by the seconds, minutes or hours. This generally needs to be
      multiplied by 6 to convert it into degrees per second/hour/minute.
      */
    property alias rotationValue: hand.rotation;

    // Set the touch/mouse grab area height and top margin
    property alias grabHeight: grabArea.height;
    property int grabMargin: units.gu(0);

    // Height and width of the clock hand
    property alias handHeight: hand.handHeight;
    property alias handWidth: hand.handWidth;

    // internal properties required for the functioning of the AnalogTouchHand
    property int timerValue: 0
    property real centerX : width / 2
    property real centerY : height / 2

    // Clock hand which lies above the rotating dial. This is stationary w.r.t to the rotating dial.
    AnalogClockHand {
        id: hand

        handHeight: units.gu(0); handWidth: units.gu(0);

        /*!
              Element to define the grab area of the clock hand. This allows us to define small clock hand
              which are still easier to grab.
              */
        Rectangle {
            id: grabArea
            color: "transparent"
            width: units.gu(5); height: width;
            anchors { top: hand.top; topMargin: touchHand.grabMargin; horizontalCenter: hand.horizontalCenter }
        }

        MouseArea{
            anchors.fill: grabArea;
            preventStealing: true;
            enabled: true;
            onPositionChanged:  {
                var point =  mapToItem (touchHand, mouse.x, mouse.y);
                var diffX = (point.x - touchHand.centerX);
                var diffY = -1 * (point.y - touchHand.centerY);
                var rad = Math.atan (diffY / diffX);
                var deg = (rad * 180 / Math.PI);

                if (diffX > 0 && diffY > 0) {
                    hand.rotation = 90 - Math.abs (deg);
                }
                else if (diffX > 0 && diffY < 0) {
                    hand.rotation = 90 + Math.abs (deg);
                }
                else if (diffX < 0 && diffY > 0) {
                    hand.rotation = 270 + Math.abs (deg);
                }
                else if (diffX < 0 && diffY < 0) {
                    hand.rotation = 270 - Math.abs (deg);
                }

                if (Math.round(hand.rotation/6) === 60) {
                    touchHand.timerValue = 0
                } else {
                    touchHand.timerValue = Math.round(hand.rotation/6);
                }
            }
        }
    }
}
