/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Juha Ristolainen <juha.ristolainen@codemancers.fi>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Nick Leppänen Larsson <frals@frals.se>
 *              Marco Biscaro <marcobiscaro2112@gmail.com>
 *              Alessandro Pozzi <signor.hyde@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import "common/ClockUtils.js" as Utils
import "clock"
import "alarm"
import "stopwatch"
import "timer"

/*!
    \brief MainView with Tabs element.
           First Tab has a single Label and
           second Tab has a single ToolbarAction.
*/

MainView {
    id: root
    // objectName for functional testing purposes (autopilot-qt5)
    objectName: "clock"

    ////////////////////////////////////////////////////////////////////////////////
    // Properties of this page
    applicationName: "ubuntu-clock-app"

    width: units.gu(50)
    height: units.gu(75)

    headerColor: "#57365E"
    backgroundColor: "#A55263"
    footerColor: "#D75669"

    ////////////////////////////////////////////////////////////////////////////////
    // New properties of this page
    property var now

    ////////////////////////////////////////////////////////////////////////////////
    // Property changed handlers

    ////////////////////////////////////////////////////////////////////////////////
    // Initialization after page has loaded
    Component.onCompleted: {
        // Any init here
        Utils.log("MainView loaded");
    }

    ////////////////////////////////////////////////////////////////////////////////
    // Helper-functions

    ////////////////////////////////////////////////////////////////////////////////
    // Non-UI elements

    // Timer that runs our app
    Timer {
        id: updateTimer
        objectName: "updateTimer"

        interval: 1000
        repeat: true
        running: true

        onTriggered: {
            now = new Date();
            // FIXME: only send callback to pages that need it
            /* TODO: this timer should probably be changed to fire
              every minute (max) when the app is in the background,
              or just be activated when the app is visible in the
              switcher
              */
            clockPage.onTimerUpdate(now);
            //alarmPage.onTimerUpdate(now);
            timerPage.onTimerUpdate(now);

            /* stopwatchPage.onTimerUpdate(now) is not required since
               there is an inbuilt stopwatch timer which runs at a
               higher frequency.
             */
        }
    }

    ////////////////////////////////////////////////////////////////////////////////
    // UI
    PageStack {
        id: pagestack

        Component.onCompleted: push(rootTabs);

        // Applications pages in tabs (Clock, Alarm, Timer, Stopwatch)
        Tabs {
            id: rootTabs
            objectName: "rootTabs"

            anchors.fill: parent

            onSelectedTabIndexChanged: {
                if (rootTabs.selectedTabIndex == 3) {
                    stopwatchLoader.sourceComponent = componentStopwatchPage;
                }
            }

            Tab {
                objectName: "ClockTab"

                title: i18n.tr("Clock")

                // Tab content begins here
                page: ClockPage {
                    id: clockPage

                    WorldClockModel {
                        id: worldModel;
                        Component.onCompleted: {
                            _loadDB();
                            Utils.log("Clock Database loaded")
                        }
                    }                    
                }
            }

            // Disabled Alarm Tab until the feature is ready for use.
            Tab {
                objectName: "AlarmTab"

                title: i18n.tr("Alarm")

                Label {
                    anchors.centerIn: parent;
                    text: "Alarm is still in development phase."
                    fontSize: "large"
                }

                //            page: AlarmPage {
                //                id: alarmPage
                //            }
            }

            Tab {
                objectName: "TimerTab"

                title: i18n.tr("Timer")

                page: TimerPage {
                    id: timerPage
                }
            }

            Tab {
                id: stopwatchTab;
                objectName: "StopwatchTab"

                title: i18n.tr("Stopwatch")

                Loader { id: stopwatchLoader; anchors.fill: parent; }

                Component {
                    id: componentStopwatchPage
                    StopwatchPage {
                        id: stopwatchPage
                    }
                }
            }
        }
    }
}
